import 'dart:convert';

import 'package:contacts_service/contacts_service.dart';
import 'package:flutter/material.dart';
import 'package:ocg_app/ocg_app.dart';
import 'package:http/http.dart' as http;
import 'package:ocg_user/src/models/contact_request.dart';
import 'package:permission_handler/permission_handler.dart';
import 'package:uuid/uuid.dart';

import '../models/user.dart';

abstract class UserRepository {
  Future<AppResponse<User>> getSelf();
  Future<AppResponse<String>> getSelfId();
  Future<AppResponse<bool>> updateSelf(User user);
  Future<AppResponse<bool>> blockUser(String userId, bool block);
  Future<AppResponse<User>> getUser(String userId);
  Future<AppResponse<List<User>>> getAllUsers();
  // Contacts
  Future<AppResponse<List<ContactRequest>>> fetchContacts();
  Future<AppResponse<bool>> addContacts(List<ContactRequest> requests);
  Future<AppResponse<List<User>>> getContacts();
}

class UserTestRepository extends UserRepository {
  static Map<String, User> _users = {};
  static String _currentId;

  @override
  Future<AppResponse<List<User>>> getAllUsers() async {
    return AppResponse.named(data: _users.values.toList());
  }

  @override
  Future<AppResponse<User>> getUser(String userId) async {
    return AppResponse.named(data: _users[userId]);
  }

  @override
  Future<AppResponse<User>> getSelf() async {
    return AppResponse.named(data: _users[_currentId]);
  }

  @override
  Future<AppResponse<String>> getSelfId() async {
    return AppResponse.named(data: _currentId);
  }

  @override
  Future<AppResponse<bool>> addContacts(List<ContactRequest> requests) {
    // TODO: implement addContacts
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<bool>> blockUser(String userId, bool block) {
    // TODO: implement blockUser
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<List<ContactRequest>>> fetchContacts() {
    // TODO: implement fetchContacts
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<List<User>>> getContacts() {
    // TODO: implement getContacts
    throw UnimplementedError();
  }

  @override
  Future<AppResponse<bool>> updateSelf(User user) {
    // TODO: implement updateSelf
    throw UnimplementedError();
  }
}

class UserNetworkRepository extends UserRepository {
  final TokenRepository tokenRepository;
  final http.BaseClient client;

  UserNetworkRepository(this.tokenRepository, this.client);

  @override
  Future<AppResponse<List<User>>> getAllUsers() async {
    return AppResponse.named(data: [
      User.named(
          id: Uuid().v4().substring(0, 24),
          name: "Bhavneet singh1",
          status: "Available",
          profilePic:
              "https://avatars3.githubusercontent.com/u/31070108?s=460&u=f2e8d6f5680f4063528d1266016c839843fd8b0e&v=4")
    ]);
  }

  @override
  Future<AppResponse<User>> getSelf() async {
    try {
      String url = "/self";
      var response = await client.get(url);
      var json = jsonDecode(response.body);
      return AppResponse.named(data: User.fromJson(json["profile"]));
    } catch (e) {
      debugPrint("Error $e");
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<String>> getSelfId() async {
    AppResponse<AppToken> response = await tokenRepository.getToken();
    if (response.isSuccess) {
      return AppResponse.named(data: response.data.userId);
    }
    return AppResponse.named(error: response.error);
  }

  @override
  Future<AppResponse<User>> getUser(String userId) async {
    try {
      String url = "/users/$userId/";
      var response = await client.get(url);
      var json = jsonDecode(response.body);
      return AppResponse.named(data: User.fromJson(json["profile"]));
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> updateSelf(User user) async {
    try {
      String url = "/users/${user.id}/";
      await client.post(url, body: jsonEncode(user.toJson()));
      return AppResponse.named(data: true);
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<bool>> blockUser(String userId, bool block) async {
    try {
      String url = "/users/$userId/${block ? "block" : "unblock"}";
      var response = await client.post(
        url,
      );
      return AppResponse.named(data: response.statusCode == 200);
    } catch (e) {
      debugPrint(
        "Error $e",
      );
      return AppResponse.named(error: e.toString());
    }
  }

  // Contacts

  @override
  Future<AppResponse<bool>> addContacts(List<ContactRequest> requests) async {
    try {
      String url = "/contacts";
      var response = await client.post(url,
          body: jsonEncode(
              {"contacts": requests.map((e) => e.toJson()).toList()}));
      return AppResponse.named(data: true);
    } catch (e) {
      debugPrint(
        "Contacts Error $e",
      );
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<List<ContactRequest>>> fetchContacts() async {
    try {
      PermissionStatus permissionStatus = await Permission.contacts.request();

      if (permissionStatus.isDenied) {
        return AppResponse.named(error: "Permission for contacts not granted");
      }
      List<Contact> contacts = (await ContactsService.getContacts(
        withThumbnails: false,
      ))
          .toList();
      Set<ContactRequest> phoneNumbers = Set<ContactRequest>();
      contacts.forEach((val) {
        val.phones.forEach((number) {
          String phone = number.value.replaceAll(" ", "");
          // Check with country code
          if (phone.startsWith("+"))
            phoneNumbers
                .add(ContactRequest.named(name: val.displayName, phone: phone));
        });
      });
      return AppResponse.named(data: phoneNumbers.toList());
    } catch (e) {
      debugPrint(
        "Contacts Error $e",
      );
      return AppResponse.named(error: e.toString());
    }
  }

  @override
  Future<AppResponse<List<User>>> getContacts() async {
    try {
      String url = "/contacts";
      var response = await client.get(
        url,
      );
      var json = jsonDecode(response.body);
      List<String> contacts = List<String>.from(json["contacts"] as List);
      Set<User> users = Set<User>();
      await Future.wait(contacts.map((val) async {
        AppResponse<User> userResponse = await getUser(val);
        if (userResponse.isSuccess) users.add(userResponse.data);
      }));
      return AppResponse.named(data: users.toList());
    } catch (e) {
      return AppResponse.named(error: e.toString());
    }
  }
}
